/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * Please see the LICENSE file or "Copyright notice" in builder.cpp for details.
 */

/*
 * OutputAsAction.cpp
 *
 *  Created on: May 8, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "Parser/FormatParserStorage.hpp"
#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "World.hpp"

#include <iostream>
#include <string>

#include <boost/filesystem/fstream.hpp>

#include "Actions/WorldAction/OutputAsAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "OutputAsAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
Action::state_ptr WorldOutputAsAction::performCall() {
  LOG(1, "Storing world to file " << params.filename.get() << ".");

  // extract suffix
  std::string FilenameSuffix;
  std::string FilenamePrefix;
  if (params.filename.get().has_filename()) {
    // get suffix
#if BOOST_VERSION >= 104600
    FilenameSuffix = params.filename.get().extension().string().substr(1); // remove the prefixed "."
    FilenamePrefix = params.filename.get().stem().string();
#else
    FilenameSuffix = params.filename.get().extension().substr(1); // remove the prefixed "."
    FilenamePrefix = params.filename.get().stem();
#endif
  } else {
    ELOG(1, "Output file does not have a suffix, cannot recognize format.");
    return Action::failure;
  }

  // store file
  // parse the file
  boost::filesystem::ofstream output;
  output.open(params.filename.get());
  if (!output.fail()) {
    FormatParserStorage::getInstance().saveWorld(output, FilenameSuffix);
  } else {
    ELOG(1, "Could not open file " << params.filename.get() << ".");
  }
  output.close();

  return Action::success;
}

Action::state_ptr WorldOutputAsAction::performUndo(Action::state_ptr _state) {
//  ParserLoadXyzState *state = assert_cast<ParserLoadXyzState*>(_state.get());

  return Action::failure;
//  string newName = state->mol->getName();
//  state->mol->setName(state->lastName);
//
//  return Action::state_ptr(new ParserLoadXyzState(state->mol,newName));
}

Action::state_ptr WorldOutputAsAction::performRedo(Action::state_ptr _state){
  return Action::failure;
}

bool WorldOutputAsAction::canUndo() {
  return false;
}

bool WorldOutputAsAction::shouldUndo() {
  return false;
}
/** =========== end of function ====================== */
